"
 COPYRIGHT (c) 1989 by Claus Gittinger
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

"{ Package: 'stx:libcomp' }"

StatementNode subclass:#ReturnNode
	instanceVariableNames:'myHome blockHome'
	classVariableNames:''
	poolDictionaries:''
	category:'System-Compiler-Support'
!

!ReturnNode class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 1989 by Claus Gittinger
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    node for parse-trees, representing return expressions
    This is a helper class for the compiler.

    [author:]
        Claus Gittinger
"
! !

!ReturnNode methodsFor:'accessing'!

expression:e
    super expression:e.

    "/ any block, which is returned cannot be inlined.
    e isBlockNode ifTrue:[
        e possiblyInlined:false
    ]

!

home:someOne blockHome:aBlockNode
    myHome := someOne.
    blockHome := aBlockNode
! !

!ReturnNode methodsFor:'code generation'!

codeForSideEffectOn:aStream inBlock:b for:aCompiler
    "redefined - drop not needed since notreached"

    ^ self codeOn:aStream inBlock:b for:aCompiler
!

codeForSimpleReturnOn:aStream inBlock:b lineNumber:lineNrOrNil for:aCompiler
    "generate code to return myself as a simple method return"

    self codeOn:aStream inBlock:b for:aCompiler.

    "as a ReturnNode, I did return, so retTop is not needed"
"/    aStream nextPut:#retTop.
!

codeLocalReturnOn:aStream inBlock:b for:aCompiler
    expression isNil ifTrue:[
        aStream nextPut:#retNil.
        ^ self.
    ].
    expression
        codeForSimpleReturnOn:aStream 
        inBlock:b 
        lineNumber:lineNr
        for:aCompiler.
!

codeOn:aStream inBlock:b for:aCompiler
    b isNil ifTrue:[
        self codeLocalReturnOn:aStream inBlock:b for:aCompiler.
        ^ self.
    ].

    "return from homeContext in a block"
    expression isNil ifTrue:[
        "/ nil-expression is not possible in smalltalk;
        "/ however, it might be in a subclass (i.e. JavaScript)
        aStream nextPut:#pushNil.
    ] ifFalse:[
        expression codeOn:aStream inBlock:b for:aCompiler.
    ].
    lineNr notNil ifTrue:[
        self codeLineNumber:lineNr on:aStream for:aCompiler
    ].

    aStream nextPut:#homeRetTop.

    "Modified: 21.10.1996 / 14:54:36 / cg"
! !

!ReturnNode methodsFor:'enumerating'!

nodeDo:anEnumerator
    "helper for parse tree walking"

    ^ anEnumerator doReturn:self value:expression

    "Modified: 19.6.1997 / 16:42:40 / cg"
! !

!ReturnNode methodsFor:'evaluation'!

evaluateExpressionIn:anEnvironment
    |val|

    val := expression evaluateIn:anEnvironment.
    myHome exitWith:val.
    "when we arrive here, the parser context is already gone
     - try block-return"
    blockHome notNil ifTrue:[blockHome exitWith:val].
    "well - what else can be done"
    ^ val
! !

!ReturnNode methodsFor:'printing & storing'!

printOn:aStream indent:i
    aStream nextPutAll:'^ '.
    expression printOn:aStream
! !

!ReturnNode methodsFor:'queries'!

isConstant
    ^ false
!

isReturnNode
    ^ true
! !

!ReturnNode class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/libcomp/ReturnNode.st,v 1.32 2005-05-04 10:03:12 cg Exp $'
! !
